from enum import Enum


class DamageType(Enum):
    """Enumeration type defining each Damage type as described in the Destiny API
    """

    NONE = 0
    KINETIC = 1
    ARC = 2
    SOLAR = 3
    VOID = 4
    RAID = 5
    STASIS = 6


class PlayerClass(Enum):
    """Enumeration type defining each player class as described in the Destiny API
    """

    TITAN = 0
    HUNTER = 1
    WARLOCK = 2
    UNKNOWN = 3


class MembershipType(Enum):
    NONE = 0
    XBOX = 1
    PSN = 2
    STEAM = 3
    BLIZZARD = 4
    STADIA = 5
    DEMON = 10
    BUNGIENEXT = 254
    All = -1


class DestinyItemSubType(Enum):
    NONE = 0
    AUTORIFLE = 6
    SHOTGUN = 7
    MACHINEGUN = 8
    HANDCANNON = 9
    ROCKETLAUNCHER = 10
    FUSIONRIFLE = 11
    SNIPERRIFLE = 12
    PULSERIFLE = 13
    SCOUTRIFLE = 14
    SIDEARM = 17
    SWORD = 18
    MASK = 19
    SHADER = 20
    ORNAMENT = 21
    FUSIONRIFLELINE = 22
    GRENADELAUNCHER = 23
    SUBMACHINEGUN = 24
    TRACERIFLE = 25
    HELMETARMOR = 26
    GAUNTLETSARMOR = 27
    CHESTARMOR = 28
    LEGARMOR = 29
    CLASSARMOR = 30
    BOW = 31
    DUMMYREPEATABLEBOUNTY = 32



import configparser
import time
from datetime import datetime, timedelta
import threading
import re
import os
import base64
import requests
from requests.packages.urllib3.exceptions import InsecureRequestWarning
from urllib.parse import urlparse
from OpenSSL import crypto
from http.server import HTTPServer, SimpleHTTPRequestHandler
import ssl
from dotenv import load_dotenv

requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

load_dotenv()

config_file_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "config", "api.ini")
config = configparser.ConfigParser()
config.read_file(open(config_file_path))
SERVER_CERTIFICATE = config['SERVER_CERTIFICATE']
XBOX_SIGN_IN = config['XBOX_SIGN_IN']

exp_urlpost = r'urlPost:\'(https://.*?)\''
exp_ppft = r'<input type="hidden" name="PPFT" id=".*" value="(.*?)"/>'
TOKEN_URL = "https://www.bungie.net/platform/app/oauth/token/"
BUNGIE_SIGN_IN_URI = "https://www.bungie.net/en/User/SignIn/Xuid?bru=%252f"

RETRIED_AUTH = RETRIED_TOKEN = RETRIED_RENEWAL = RETRIED_SERVER = False


class OAuth2:

    def __init__(self, client_id: str, client_secret: str):
        ENCODED_DATA = base64.b64encode(bytes(f"{client_id}:{client_secret}", "ISO-8859-1")).decode("ascii")
        self.__headers = {
            "Authorization": "Basic " + ENCODED_DATA,
            "Content-Type": "application/x-www-form-urlencoded"
        }
        self.__data = None
        self.__httpd = HTTPServer(('127.0.0.1', 5000), SimpleHTTPRequestHandler)
        os.environ["CLIENT_ID"] = client_id
        os.environ["CLIENT_SECRET"] = client_secret
        self.__cert_gen()
        self.__start_server()
        self.__get_authorization_code()
        self.__get_tokens()
        self._enabled = True
        self.__renewal_thread = threading.Thread(target=self.__renew_tokens)
        self.__renewal_thread.start()

    @staticmethod
    def __cert_gen():
        serial_number = 0
        validity_end_in_seconds = 10 * 365 * 24 * 60 * 60
        k = crypto.PKey()
        k.generate_key(crypto.TYPE_RSA, 4096)
        cert = crypto.X509()
        cert.get_subject().C = SERVER_CERTIFICATE["COUNTRY_NAME"]
        cert.get_subject().ST = SERVER_CERTIFICATE["STATE_OR_PROVINCE_NAME"]
        cert.get_subject().L = SERVER_CERTIFICATE["LOCALITY_NAME"]
        cert.get_subject().O = SERVER_CERTIFICATE["ORGANIZATION_NAME"]
        cert.get_subject().OU = SERVER_CERTIFICATE["ORGANIZATION_UNIT_NAME"]
        cert.get_subject().CN = SERVER_CERTIFICATE["COMMON_NAME"]
        cert.get_subject().emailAddress = SERVER_CERTIFICATE["EMAIL_ADDRESS"]
        cert.set_serial_number(serial_number)
        cert.gmtime_adj_notBefore(0)
        cert.gmtime_adj_notAfter(validity_end_in_seconds)
        cert.set_issuer(cert.get_subject())
        cert.set_pubkey(k)
        cert.sign(k, 'sha512')
        with open("server.pem", "w") as f:
            f.write(crypto.dump_certificate(crypto.FILETYPE_PEM, cert).decode("utf-8"))
        with open("server.pem", "a") as f:
            f.write(crypto.dump_privatekey(crypto.FILETYPE_PEM, k).decode("utf-8"))

    def __update_post_data(self):
        self.__data = {
            'grant_type': "authorization_code",
            'code': os.getenv("AUTH_CODE"),
        }

    def __start_server(self):
        try:
            self.__httpd.socket = ssl.wrap_socket(self.__httpd.socket, certfile='server.pem', server_side=True)
            self.server_thread = threading.Thread(target=self.__httpd.serve_forever)
            self.server_thread.start()
        except:
            global RETRIED_SERVER
            if not RETRIED_SERVER:
                RETRIED_SERVER = True
                with open('err.log', 'a') as f:
                    f.write(f'WARNING Server startup failed! {datetime.now().strftime("%m/%d/%Y at %H:%M:%S")} \n')
                self.__cert_gen()
                self.__start_server()

    def __get_authorization_code(self):
        try:
            if not self.server_thread.is_alive():
                self.__start_server()
            s = requests.Session()
            r = s.get(BUNGIE_SIGN_IN_URI)
            url_post = re.findall(exp_urlpost, r.content.decode())[0]
            ppft = re.findall(exp_ppft, r.content.decode())[0]
            payload = {'login': XBOX_SIGN_IN["XBOX_EMAIL"], 'passwd': XBOX_SIGN_IN["XBOX_PASSWORD"], 'PPFT': ppft}
            s.post(url_post, data=payload)
            api_headers = {'X-API-Key': os.environ["X-API-KEY"], 'x-csrf': s.cookies.get_dict()['bungled']}
            r = s.get(
                f'https://www.bungie.net/en/OAuth/Authorize?client_id={os.environ["CLIENT_ID"]}&response_type=code',
                headers=api_headers,
                verify=False)
            os.environ["AUTH_CODE"] = urlparse(url=r.url).query[5:]
            self.__httpd.shutdown()
        except:
            global RETRIED_AUTH
            if not RETRIED_AUTH:
                RETRIED_AUTH = True
                with open('err.log', 'a') as f:
                    f.write(
                        f'WARNING Unable to acquire authentication code! {datetime.now().strftime("%m/%d/%Y at %H:%M:%S")}\n')
                self.__get_authorization_code()

    def __get_tokens(self):
        try:
            self.__update_post_data()
            response = requests.post(url=TOKEN_URL, headers=self.__headers, data=self.__data)
            response = response.json()
            os.environ["ACCESS_TOKEN"] = response['access_token']
            os.environ["REFRESH_TOKEN"] = response['refresh_token']
        except:
            global RETRIED_TOKEN
            if not RETRIED_TOKEN:
                RETRIED_TOKEN = True
                with open('err.log', 'a') as f:
                    f.write(f'WARNING Unable to acquire tokens! {datetime.now().strftime("%m/%d/%Y at %H:%M:%S")}\n')
                self.__get_authorization_code()
                self.__get_tokens()

    def __renew_tokens(self):
        while self._enabled:
            token_expiration = datetime.now() + timedelta(minutes=59)
            while datetime.now() < token_expiration and self._enabled:
                time.sleep(5)
            try:
                self.__update_post_data()
                response = requests.post(url=TOKEN_URL, headers=self.__headers, data=self.__data)
                response = response.json()
                os.environ["ACCESS_TOKEN"] = response['access_token']
                os.environ["REFRESH_TOKEN"] = response['refresh_token']
            except:
                global RETRIED_RENEWAL
                if not RETRIED_RENEWAL:
                    RETRIED_RENEWAL = True
                    with open('err.log', 'a') as f:
                        f.write(f'WARNING Unable to renew tokens! {datetime.now().strftime("%m/%d/%Y at %H:%M:%S")}\n')
                    self.__get_authorization_code()
                    self.__get_tokens()



Profiles = "100"
VendorReceipts = "101"
ProfileInventories = "102"
ProfileCurrencies = "103"
ProfileProgression = "104"
PlatformSilver = "105"
Characters = "200"
CharacterInventories = "201"
CharacterProgressions = "202"
CharacterRenderData = "203"
CharacterActivities = "204"
CharacterEquipment = "205"
ItemInstances = "300"
ItemObjectives = "301"
ItemPerks = "302"
ItemRenderData = "303"
ItemStats = "304"
ItemSockets = "305"
ItemTalentGrids = "306"
ItemCommonData = "307"
ItemPlugStates = "308"
ItemPlugObjectives = "309"
ItemReusablePlugs = "310"
Vendors = "400"
VendorCategories = "401"
VendorSales = "402"
Kiosks = "500"
CurrencyLookups = "600"
PresentationNodes = "700"
Collectibles = "800"
Records = "900"
Transitory = "1000"
Metrics = "1100"



import os
import requests
from .OAuth2 import OAuth2
from .destiny_enums import MembershipType

API_ROOT_PATH = "https://www.bungie.net/Platform"


class BungieAPI:
    def __init__(self, api_key: str):
        self.__HEADERS = {"X-API-Key": api_key, "Authorization": "Bearer "}
        self.__OAuth2 = None
        os.environ["X-API-KEY"] = api_key

    def start_oauth2(self, client_id, client_secret):
        self.__OAuth2 = OAuth2(client_id, client_secret)

    def close_oauth2(self):
        if self.__OAuth2 is not None:
            self.__OAuth2._enabled = False
        else:
            print("OAuth2 is not enabled")

    def __renew_headers(self):
        self.__HEADERS["Authorization"] = "Bearer " + os.getenv("ACCESS_TOKEN")

    def get_bungie_user_by_id(self, membership_id: int) -> dict:
        self.__renew_headers()
        api_call = requests.get(API_ROOT_PATH + "/User/GetBungieNetUserById/" + str(membership_id) + "/",
                                headers=self.__HEADERS)
        return (api_call.json())['Response']

    def search_users(self, query_string: str) -> dict:
        self.__renew_headers()
        api_call = requests.get(API_ROOT_PATH + "/User/SearchUsers/?q=" + query_string, headers=self.__HEADERS)
        return (api_call.json())['Response']

    def get_profile(self, membership_type: MembershipType, membership_id: int) -> dict:
        self.__renew_headers()
        api_call = requests.get(API_ROOT_PATH + "/Destiny2/" + str(membership_type.value) + "/Profile/"
                                + str(membership_id) + "/?components=100", headers=self.__HEADERS)
        return (api_call.json())['Response']

    def get_clan_weekly_reward_state(self, group_id: int) -> dict:
        self.__renew_headers()
        api_call = requests.get(API_ROOT_PATH + "/Destiny2/Clan/" + str(group_id) + "/WeeklyRewardState/",
                                headers=self.__HEADERS)
        return (api_call.json())['Response']

    def get_collectible_node_details(self, membership_type: MembershipType, membership_id: int,
                                     character_id: int,
                                     collectible_presentation_node_hash: int, components: str) -> dict:
        self.__renew_headers()
        api_call = requests.get(API_ROOT_PATH + "/Destiny2/" + str(membership_type.value) + "/Profile/"
                                + str(membership_id) + "/Character/" + str(character_id)
                                + "/Collectibles/" + str(
            collectible_presentation_node_hash) + "/?components=" + components,
                                headers=self.__HEADERS)
        return (api_call.json())['Response']

    def get_linked_profiles(self, membership_type: MembershipType, membership_id: int):
        self.__renew_headers()
        api_call = requests.get(API_ROOT_PATH + "/Destiny2/" + str(membership_type.value) + "/Profile/"
                                + str(membership_id) + "/LinkedProfiles/", headers=self.__HEADERS)
        return (api_call.json())['Response']

    def get_character(self, membership_type: MembershipType, membership_id: int, character_id: int) -> dict:
        self.__renew_headers()
        api_call = requests.get(API_ROOT_PATH + "/Destiny2/" + str(membership_type.value) + "/Profile/"
                                + str(membership_id) + "/Character/" + str(character_id) + "/?components=200",
                                headers=self.__HEADERS)
        return (api_call.json())['Response']

    def get_vendor(self, membership_type: MembershipType, membership_id: int, character_id: int,
                   vendor_hash: str, components: str):
        self.__renew_headers()
        api_call = requests.get(API_ROOT_PATH + "/Destiny2/" + str(membership_type.value) + "/Profile/"
                                + str(membership_id) + "/Character/" + str(character_id)
                                + "/Vendors/" + vendor_hash + "/?components=" + components, headers=self.__HEADERS)
        return (api_call.json())['Response']

    def get_vendors(self, membership_type: MembershipType, membership_id: int, character_id: int,
                    components: str):
        self.__renew_headers()
        api_call = requests.get(API_ROOT_PATH + "/Destiny2/" + str(membership_type.value) + "/Profile/"
                                + str(membership_id) + "/Character/" + str(character_id)
                                + "/Vendors/?components=" + components, headers=self.__HEADERS)
        return (api_call.json())['Response']

    def manifest(self, entity_type: str, hash_identifier: str) -> dict:
        """ Manifests the specified entity. Returns general information about the entity, see API documentation for more details:
            https://bungie-net.github.io/#Destiny2.GetDestinyEntityDefinition

        :param entity_type: The type of entity for whom you would like results. These correspond to the entity's
            definition contract name. See api.ini/Definitions
        :param hash_identifier: The hash identifier for the specific Entity you want returned.
        :return: dict
        """
        self.__renew_headers()
        api_call = requests.get(API_ROOT_PATH + "/Destiny2/Manifest/" + entity_type + "/" + hash_identifier,
                                headers=self.__HEADERS)
        return (api_call.json())['Response']

    def get_public_vendors(self, components: str) -> dict:
        """Returns information on the requested vendor, see API documentation for more details:
            https://bungie-net.github.io/#Destiny2.GetPublicVendors

        :param vendor_hash: The hash identifier for the specific Vendor you want returned.
        :param components: See api.ini/Components
        :return: dict
        """
        self.__renew_headers()
        api_call = requests.get(API_ROOT_PATH + '/Destiny2//Vendors/?components=' + components,
                                headers=self.__HEADERS)
        return (api_call.json())['Response']

    def search_entities(self, entity_type: str, search_term: str) -> dict:
        """Searches for the specified entity in the API, see API documentation for more details:
            https://bungie-net.github.io/#Destiny2.SearchDestinyEntities

        :param entity_type: The type of entity for whom you would like results. These correspond to the entity's
            definition contract name. See api.ini/Definitions
        :param search_term: The string to use when searching for Destiny entities.
        :return: dict
        """
        self.__renew_headers()
        api_call = requests.get(API_ROOT_PATH + '/Destiny2/Armory/Search/' + entity_type + "/" + search_term + "/",
                                headers=self.__HEADERS)
        return (api_call.json())['Response']

    def search_destiny_player(self, membership_type: MembershipType, display_name: str):
        self.__renew_headers()
        api_call = requests.get(
            API_ROOT_PATH + '/Destiny2/SearchDestinyPlayer/' + str(membership_type.value) + "/" + display_name + "/",
            headers=self.__HEADERS)
        return (api_call.json())['Response']



import setuptools

setuptools.setup(
    name="pybungie",
    version="0.5.4",
    author="Hayden Cole",
    author_email="cole.haydenj@gmail.com",
    description="For interacting with the Bungie.net API",
    url="https://github.com/Hayden-J-C/pybungie.git",
    packages=['pybungie'],
    package_data={'pybungie': ['config/*', ]},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=['requests', 'pyOpenSSL', 'python-dotenv'],
    python_requires='>=3.6',
)
